//  Adafruit Feather M0 Basic Proto example code
//  based on Arduino Nano example code by Emanuel FERU: https://github.com/EmanuelFeru/hoverboard-firmware-hack-FOC/blob/master/Arduino/hoverserial/hoverserial.ino

#define HOVER_SERIAL_BAUD   115200      // [-] Baud rate for HoverSerial (used to communicate with the hoverboard)
#define SERIAL_BAUD         115200      // [-] Baud rate for built-in Serial (used for the Serial Monitor)
#define START_FRAME         0xABCD      // [-] Start frame definition for reliable serial communication
#define TIME_SEND           100         // [ms] Sending time interval
//#define DEBUG_RX                        // [-] Debug received data. Prints all bytes to serial (comment-out to disable)

uint8_t idx = 0;                        // Index for new data pointer
uint16_t bufStartFrame;                 // Buffer Start Frame
byte *p;                                // Pointer declaration for the new received data
byte incomingByte;
byte incomingBytePrev;

typedef struct {
  uint16_t start;
  int16_t  steer;
  int16_t  speed;
  uint16_t checksum;
} SerialCommand;
SerialCommand Command;

typedef struct {
  uint16_t start;
  int16_t  cmd1;
  int16_t  cmd2;
  int16_t  speedR_meas;
  int16_t  speedL_meas;
  int16_t  batVoltage;
  int16_t  boardTemp;
  uint16_t cmdLed;
  uint16_t checksum;
} SerialFeedback;
SerialFeedback Feedback;
SerialFeedback NewFeedback;

unsigned long TimeSend = 0;


// ########################## SETUP ##########################
void setup()
{
  Serial.begin(SERIAL_BAUD);
  Serial.println("Hoverboard Serial v1.0");

  Serial1.begin(HOVER_SERIAL_BAUD);  // using the hardware Serial1 TX/RX pins instead of SoftwareSerial
  pinMode(LED_BUILTIN, OUTPUT);
}


// ########################## LOOP ##########################

void loop(void)
{
  Receive();

  unsigned long timeNow = millis();
  if (TimeSend > timeNow) return;  // Send commands every 100ms
  TimeSend = timeNow + TIME_SEND;

  int Speed = 100;
  int Steer = 0;

  // some boards need a zero after power on to start the motors
  if (Speed > 0 && Feedback.speedR_meas == 0) {Send(0, 0); delay (10);}

  Send(Steer, Speed);  // <---------------------------------------------- Send(Steer, Speed); -1000 bis +1000

  digitalWrite(LED_BUILTIN, (timeNow % 200) < 100);


}



// ########################## SEND ##########################
void Send(int16_t uSteer, int16_t uSpeed)
{
  // Create command
  Command.start    = (uint16_t)START_FRAME;
  Command.steer    = (int16_t)uSteer;
  Command.speed    = (int16_t)uSpeed;
  Command.checksum = (uint16_t)(Command.start ^ Command.steer ^ Command.speed);

  // Write to Serial
  Serial1.write((uint8_t *) &Command, sizeof(Command));
}


// ########################## RECEIVE ##########################
void Receive()
{
  // Check for new data availability in the Serial buffer
  if (Serial1.available()) {
    incomingByte    = Serial1.read();                                 // Read the incoming byte
    bufStartFrame = ((uint16_t)(incomingByte) << 8) | incomingBytePrev;   // Construct the start frame
  }
  else {
    return;
  }

  // If DEBUG_RX is defined print all incoming bytes
#ifdef DEBUG_RX
  Serial.print(incomingByte);
  return;
#endif

  // Copy received data
  if (bufStartFrame == START_FRAME) {                     // Initialize if new data is detected
    p     = (byte *)&NewFeedback;
    *p++  = incomingBytePrev;
    *p++  = incomingByte;
    idx   = 2;
  } else if (idx >= 2 && idx < sizeof(SerialFeedback)) {  // Save the new received data
    *p++  = incomingByte;
    idx++;
  }

  // Check if we reached the end of the package
  if (idx == sizeof(SerialFeedback)) {
    uint16_t checksum;
    checksum = (uint16_t)(NewFeedback.start ^ NewFeedback.cmd1 ^ NewFeedback.cmd2 ^ NewFeedback.speedR_meas ^ NewFeedback.speedL_meas
                          ^ NewFeedback.batVoltage ^ NewFeedback.boardTemp ^ NewFeedback.cmdLed);

    // Check validity of the new data
    if (NewFeedback.start == START_FRAME && checksum == NewFeedback.checksum) {
      // Copy the new data
      memcpy(&Feedback, &NewFeedback, sizeof(SerialFeedback));

      // Print data to built-in Serial
      //      Serial.print(" SteerSend: "); Serial.print(Feedback.cmd1);
      //      Serial.print(" SpeedSend: "); Serial.print(Feedback.cmd2);
      Serial.print(Feedback.speedR_meas);       Serial.print(" r  ");
      Serial.print(Feedback.speedL_meas);       Serial.print(" l  ");
      Serial.print(Feedback.batVoltage / 100.); Serial.print("v  ");
      Serial.print(Feedback.boardTemp / 10.);   Serial.print("°  ");
      Serial.println(Feedback.cmdLed); //?
    } else {
      Serial.println("Non-valid data skipped");
    }
    idx = 0;  // Reset the index (it prevents to enter in this if condition in the next cycle)
  }

  // Update previous states
  incomingBytePrev  = incomingByte;
}
